import click

from FrictionlessDarwinCore import DwCArchive

@click.command()
@click.argument('dwca', type=str, required=True)
@click.argument('outpath', type=click.Path(writable=True),required=True)
@click.option('-f','--format', help='Output format', type=click.Choice(['json', 'md']))
def cli(dwca, outpath, format):
        da = DwCArchive(dwca)
        da.infer()
        if  format is None:
            da.save(outpath)
        if format == 'json':
            da.to_json(outpath)
        if format == 'md':
            da.to_markdown(outpath)

if __name__ == '__main__':
    cli()



import csv
import os


class DwCVocabulary:

    def __init__(self, path=None):
        self.dwc_dictionary = dict()
        if path is None:
            path = os.path.join(os.path.dirname(__file__), 'fdwc_terms.csv')
        with open(path, newline='') as csvFile:
            reader = csv.DictReader(csvFile)
            for row in reader:
                namespace = row['namespace'].lower()
                if namespace in self.dwc_dictionary:
                    ns = self.dwc_dictionary[namespace]
                else:
                    ns = dict()
                    self.dwc_dictionary[row['namespace'].lower()] = ns
                term = dict()
                for x, y in row.items():
                    term[x] = y
                ns[row['name'].lower()] = term

    def term(self, qualified_name):
        tokens = qualified_name.lower().rsplit("/", 1)
        if tokens[0] in self.dwc_dictionary:
            ns = self.dwc_dictionary[tokens[0]]
            if tokens[1] in ns:
                return ns[tokens[1]]
        return None

    def size(self):
        result = 0
        for k, ns in self.dwc_dictionary.items():
            result = result + len(ns)
        return result



from FrictionlessDarwinCore import *
import sys
import shutil
import requests
import zipfile
import tempfile


class DwCArchive:
    voc = DwCVocabulary()

    def __init__(self, path_or_url):
        self.valid = True
        self.dwca = path_or_url
        self.metadata = None
        self.structure = None
        self.tf = None
        if self.dwca.startswith('http'):
            self.path = self.download()
        else:
            self.path = self.dwca

    def download(self):
        #  download DwCArchive into temporary file
        self.tf = tempfile.NamedTemporaryFile()
        try:
            response = requests.get(self.dwca)
            self.tf.write(response.content)
        except BaseException:
            print('HTTP download failed')
            self.valid = False
        else:
            print('downloading ' + self.dwca + ' as ' + self.tf.name)
        return self.tf.name

    def infer(self):
        if zipfile.is_zipfile(self.path):
            self.load()
        else:
            self.valid = False
            print('dwca is not a zipfile')

    def load(self):
        eml = ''
        meta = ''
        zf = zipfile.ZipFile(self.path, mode='r')
        try:
            for info in zf.infolist():
                if info.filename == 'eml.xml':
                    eml = zf.read(info.filename).decode()
                if info.filename == 'metadata.xml':
                    eml = zf.read(info.filename).decode()
                if info.filename == 'meta.xml':
                    meta = zf.read(info.filename).decode()
            if eml != '' and meta != '':
                self.metadata = DwCMetadata(eml)
                self.structure = DwCStructure(meta, eml)
                self.metadata.convert()
                self.structure.convert()
                self.valid = self.metadata.valid and self.structure.valid
        except BaseException:
            print(sys.exc_info())
            print('load zip failed')
            self.valid = False
        finally:
            zf.close()

    def save(self, output):
        # first copy zipfile to output
        shutil.copyfile(self.path, output)
        zf = zipfile.ZipFile(output, mode='a')
        try:
            # Add a README.md file that describes the package
            zf.writestr('readme.md', self.metadata.as_markdown())
            # Add a datapackage.json
            zf.writestr('datapackage.json', self.structure.as_json())
        except IOError as e:
            print("Unable to copy file. %s" % e)
        else:
            print('saving zipfile to ' + output)
        finally:
            zf.close()

    def to_json(self, output):
        o = open(output, 'w')
        try:
            o.write(self.structure.as_json())
        except IOError as e:
            print("Unable to write file. %s" % e)
        else:
            print('saving json to ' + output)
        finally:
            o.close()

    def to_markdown(self, output):
        o = open(output, 'w')
        try:
            o.write(self.metadata.as_markdown())
        except IOError as e:
            print("Unable to write file. %s" % e)
        else:
            print('saving markdown to ' + output)
        finally:
            o.close()
